# JS Runner

# Imports
from subprocess import call as command
from os import unlink

# Errors
from .errors import (
    AlredyANPMPackageError,
    NodeJSNotFoundError,
    FailedToCreateFile,
    NonZeroExitCode,
)


class javascript:
    def __init__(self):
        pass

    def check_node(self):
        try:
            command("node -v", shell=True)

        except:
            raise NodeJSNotFoundError(
                "node.js not found. make sure node.js is installed and added to path."
            )

    def run(self, code: str):
        try:
            temp_file = open("pyjstempfile.js", "w")
            temp_file.write(code)
            temp_file.close()
        except:
            raise FailedToCreateFile("failed to create temporary javascript file.")

        node_exit_status = command("node pyjstempfile.js", shell=True)
        if node_exit_status != 0:
            raise NonZeroExitCode(f"node.js exited with error code: {node_exit_status}")

        unlink("./pyjstempfile.js")



from subprocess import call as command
from .errors import NonZeroExitCode, NPMNotFoundError, AlredyANPMPackageError
from os import chdir, getcwd, mkdir, unlink, listdir
from shutil import rmtree


def run_command(command_to_run: str):
    status = command(command_to_run, shell=True)
    if status != 0:
        raise NonZeroExitCode(f"npm exited with error code: {status}")


class npm:
    def __init__(self):
        pass

    def install(self):
        run_command("npm install")

    def install_packages(self, *package_name: str, npm_args: str = ""):
        run_command("npm " + npm_args + " install " + "".join(package_name))

    def uninstall(self, *packages: str):
        run_command("npm uninstall " + "".join(packages))

    def check_npm(self):
        try:
            command("npm -v", shell=True)
        except:
            raise NPMNotFoundError(
                "npm not found. make sure npm is installed and added to path."
            )

    def init_npm_package(
        self,
        name: str,
        license: str = "MIT",
        author: str = "",
        description: str = "",
        version: str = "1.0.0",
        main: str = "index.js",
        npm_package_path: str = "./",
    ):
        package_json = """{
    "name": "<name>",
    "version": "<version>",
    "description": "<description>",
    "main": "<main>",
    "scripts": {
    "test": "echo \\\"Error: no test specified\\\" && exit 1"
},
    "keywords": [],
    "author": "<author>",
    "license": "<license>"
}
"""
        package_json = package_json.replace("<name>", name)
        package_json = package_json.replace("<version>", version)
        package_json = package_json.replace("<description>", description)
        package_json = package_json.replace("<main>", main)
        package_json = package_json.replace("<author>", author)
        package_json = package_json.replace("<license>", license)
        if self.check_if_npm_package(npm_package_path):
            raise AlredyANPMPackageError(f"{npm_package_path}: alredy a npm package.")

        else:
            old_pwd = getcwd()
            chdir(npm_package_path)
            package_json_file = open("package.json", "w")
            package_json_file.write(package_json)
            mkdir("./node_modules")
            chdir(old_pwd)

    def remove_npm_package(self, npm_package_path: str = "./"):
        old_pwd = getcwd()
        chdir(npm_package_path)
        rmtree("node_modules")
        unlink("./package.json")
        try:
            unlink("./package-lock.json")
        except:
            pass
        chdir(old_pwd)

    def check_if_npm_package(self, npm_package_path: str = "./"):
        dirs = listdir(npm_package_path)
        if "node_modules" in dirs and "package.json" in dirs:
            return True

        else:
            return False

    def npm_cli(self, args: str):
        run_command("npm " + args)



from setuptools import setup, find_packages
import codecs

with codecs.open("README.md", encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = "1.0.0"
DESCRIPTION = "Execute JavaScript Code From Python."

# Setting up
setup(
    name="py_js_runner",
    version=VERSION,
    author="Yasir",
    author_email="<yasirsadman@outlook.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=[],
    keywords=["python", "javascript", "js", "runjsfrompython"],
)



# Errors


class NodeJSNotFoundError(Exception):
    pass


class NPMNotFoundError(Exception):
    pass


class FailedToCreateFile(Exception):
    pass


class NonZeroExitCode(Exception):
    pass


class AlredyANPMPackageError(Exception):
    pass


class RuntimeNotFoundError(Exception):
    pass


class PlatformError(Exception):
    pass
